#include "asm86.h" 
#include "ti86asm.inc" 
#include ti86abs.inc

road = $C128 ; 64 byte array to store position of road in each row.
_flushallmenus  equ     49DCh   ; clear the 86's menu key stacks
_dispAHL = $4a33
_homeup         equ       4A95h       ; cursror to top left of home screen


.org _asm_exec_ram
;----------------------
	nop
	jp Startup
.dw 0000                 ;Ashell stuff
.dw title
title:
.db "Big, Bad Racer by Matt Teiken",0
;----------------------
Startup:
 ld a,1
 ld (rows),a
 call BUSY_OFF
 call _flushallmenus
 call _clrLCD
 ld hl,TitleScreen
 ld de,$fc00
 call DispRLE
 call pause
 call _clrLCD
 ld a,40
 ld (width),a

 sbc hl,hl	
 ld (score),hl
 ld a,60
 ld (cx),a


 ld c,0
 xor a
 ld (temp),a
 call ClearRoad
 ld c,0
 call DisplayRoad
 call LoadCar

MainLoop:
 ld c,0
 call DisplayRoad
 ld a,63
 ld (temp),a
 call MoveRoad

 call GetNewData
 ld hl,road
 ld (hl),a

 ld c,0
 call DisplayRoad

 call GetInput

 
 
 ld a,$BF
 out (1),a
 nop
 nop
 in a,(1)
 bit 6,a	       ; Check if Exit has been pressed
 jp z,ExitGame

 ld e,56
 call CheckCrash
 cp 5
 jp z,ExitGame

 ld hl,(score)
 inc hl
 ld (score),hl
 ld de,3762h
 call SdispHL
 ld hl,(score)
 ld a,(temp3)
 ld b,a
 ld a,h
 ld (temp3),a
 cp b
 jp z,contchk
 call narrowerR

contchk:
 ld a,h
 cp 250
 jp z,contck
 jp MainLoop
contck:
 ld hl,(score)
 ld a,l
 cp 250
 jp z,OutOfGas

 jp MainLoop

narrowerR:
 ld c,0
 call DisplayRoad
 ld a,(width)
 dec a
 ld (width),a
 ld c,0
 call DisplayRoad
 ret

OutOfGas:
 ld de, 2025h
 ld hl,gastxt
 call DispText
 jp ExitGame


CheckCrash:
 ld d,0

 ld hl,road
 add hl,de
 ld a,(hl)
 ld c,a
 add a,8
 ld h,a
 ld a,c
 ld a,(width)
 add a,h
 sub 7
 ld l,a
 ld a,(cx)
 ld c,a
 cp h
 jp p,conttest
 jp Crash
conttest:
 ld a,c
 add a,8
 cp l
 jp p,Crash
 inc e
 ld a,e
 cp 64
 jp nz,CheckCrash
 xor a
 ret

Crash:
 call LoadCar
 ld a,8
 ld (rows),a
 call BurnCar
 ld de, 2025h
 ld hl,crashtxt
 call DispText
 ld a,5
 ret

BurnCar:
 ld a,(temp)
 inc a
 ld (temp),a
 cp 10
 jp z,break
 call Burn1
 ld b,5
 call Delay
 call Burn1
 call Burn2
 ld b,5
 call Delay
 call Burn2
 jp BurnCar

Burn1:
 ld hl,Burned1
 ld a,(cx)
 ld b,a
 ld a,56
 ld c,a
 call PutSprite
 ret

Burn2:
 ld hl,Burned2
 ld a,(cx)
 ld b,a
 ld a,56
 ld c,a
 call PutSprite
 ret

Delay:
 ld a,30
Delay2:
 dec a
 cp 0
 jp z,Delay3
 jp Delay2
Delay3:
 ld a,b
 dec a
 ld b,a
 cp 0
 jp nz,Delay
 ret


GetInput:
 ld a,$FE
 out (1),a
 nop
 nop
 in a,(1)
 bit 2,a	      
 jp nz,CheckLeft
 call MoveRight
CheckLeft:
 ld a,$FE
 out (1),a
 nop
 nop
 in a,(1)      
 bit 1,a	      
 jp nz,break
 call MoveLeft
 ret

MoveLeft:
 ld a,(cx)
 cp 1
 jp z,break
 call LoadCar
 ld a,(cx)
 dec a
 ld (cx),a
 call LoadCar
 ret
MoveRight:
 ld a,(cx)
 cp 120
 jp z,break
 call LoadCar
 ld a,(cx)
 inc a
 ld (cx),a
 call LoadCar
 ret

LoadCar:
 ld a,8
 ld (rows),a
 ld a,(cx)      ;load x variable into a
 ld b,a       
 ld c,56 
 ld hl,CarPic   
 call PutSprite ;Put Pic at (c,b)
 ld a,1
 ld (rows),a
 ret




GetNewData:
 ld hl,road
 inc hl
 ld a,(hl)
 ld (temp2),a
 ld hl,road
 ld a,4;3
 call random
 ;inc a
 ld (temp),a
 ld a,2
 call random
 inc a
 cp 1
 jp nz,New2
New1:
 ld a,(temp2)
 cp 5
 jp p,Cont1
 jp New2
Cont1:
 ld a,(temp)
 ld b,a
 ld a,(temp2)
 sub b
 ret

New2:
 ld a,(temp2)
 ld c,a
 ld a,(width)
 add a,c
 cp 90
 jp p,New1
 ld a,(temp)
 ld b,a
 ld a,(temp2)
 add a,b
 ret


ExitGame:
 call pause
 call _clrLCD
 call Highscores
 call pause
 call _clrLCD
 call _homeup
 ret

MoveRoad:
 ld d,0
 ld e,62
 ld hl,road
 add hl,de
MoveRoad2:
 ld a,(hl)
 inc hl
 ld (hl),a
 dec hl
 dec hl
 ld a,(temp)
 inc a
 ld (temp),a
 cp 63
 jp z,break
 jp MoveRoad2

DisplayRoad:
 ld hl,road
 ld d,0
 ld e,c
 add hl,de
 ld a,(hl)
 ld b,a
 ld (temp2),a
 ld a,c
 ld (temp),a

 ld hl,Pic
 call PutSprite

 ld a,(temp)
 ld c,a
 ld a,(width)
 ld b,a
 ld a,(temp2)
 add a,b
 ld h,a

 ld b,h
 ld hl,Pic
 call PutSprite

 ld a,(temp)
 ld c,a
 inc c
 ld a,c
 cp 64
 jp z,break
 jp DisplayRoad
 
ClearRoad:
 ld hl,road
 ld d,0
 ld e,c
 add hl,de
 ld (hl),50
 inc c
 ld a,c
 cp 64
 jp z,break
 jp ClearRoad

break:
 ret
 
temp:
.db 0

width:
.db 25

temp2:
.db 0

cx:
.db 52

score:
.dw 0

High:
.dw 0

rows:
.db 1

temp3:
.db 0

crashtxt:
.db "You Crashed",0

gastxt:
.db "Out of gas!!!",0

HighScr:	
.db "High "

scoretxt:	
.db "Score: ",0

NHSMsg:
.db "New High Score!",0

varname:
.db 5,"Racer"


Highscores:
 ld hl,(High)
 call dispHigh
CheckHigh:
 ld de,(High)
 ld hl,(score)
 call scorechk
 or a
 jp nz,break
 ld (High),hl
	ld hl,varname-1
	rst 20h
	rst 10h
	ld a,b
	ex de,hl
	call _load_ram_ahl
	ld de,High-$d744
	add hl,de
	ld de,(High)
	ld (hl),e
	inc hl
	ld (hl),d
 ld hl,1010h
 ld (_penCol),hl
 ld hl,NHSMsg
 call _vputs
 ret

dispHigh:
 ld de,$0610
 push hl
 ld hl,HighScr
 call DispText
 ld de,0634h
 pop hl
 call SdispHL
show_sc:
 ld de, 1810h
 ld hl,scoretxt
 call DispText
 ld de,1825h
 ld hl,(score)
 call SdispHL
 call GET_KEY			; flush keypress buffer
 ret

;---------= High Score =---------
; Input: de=previous high score
;	hl=current score
; Output: hl=high score
;	z=1 (a=0) if new high score, z=0 (a=1) if not
; Registers destroyed: af, de, hl
scorechk:
	push	hl
	xor	a
	sbc	hl,de
	pop	hl
	jr	z,nnhs
	jr	nc,nhs
nnhs:	ex	de,hl
	inc	a
nhs:	or	a
	ret



pause:
    call GET_KEY             ;gets the recent key pressed
    cp K_EXIT                ;compare it to exit
    jp z,break         ;if it's exit, go to TimetoLeave
    cp K_SECOND               ;and similar
    jp z,break
    cp K_ENTER               ;and similar
    jp z,break
    jp pause

SdispHL:
 		ld (_penCol),de			
		ld de,-1				;routine by Matthew Scepcar
		ld (_curRow),de
		xor a
		call $4a33
		dec hl
		jp _vputs

_clrbuf:
        ld hl,$ca06
        ld de,$ca07
        ld bc,1023
        ld (hl),0
        ldir
        ret

DispText:
 ld (_penCol), de
 call _vputs
 ret

random: 	       ; Creates a random number 0 <= x < A
 	push bc
 	push de
 	push hl
 	ld b,a
 	ld a,r
 	add a,a
 	ld hl,0
 	ld d,0
 	ld e,a
RMul:
 	add hl,de
 	djnz RMul
 	ld a,h
 	pop hl
 	pop de
 	pop bc
 	ret

PutSprite: 
;FastSprite_______________________________
;Input: Sprite in (HL), (b,c)            |
;Output: Sprite xor'ed onto the screen   |
;Destroyed: none                         |
;-----------------------------------------
	push af
	push bc
	push de
	push ix
	push hl
	push hl                 ;\
	pop ix                  ; > hl -> ix
	ld hl, FSByte           ;> point to Tempbyte
	ld (hl), c              ;> put c into Byte for later use
	ld a, b
	rrca                    ;\
	rrca                    ; > divide x by 8 in accumulator
	rrca                    ;/                      
	rld                     ;> Rotate left 4 bytes  
	or $FC                  ;> add a and $FC
	ld l, (hl)              ;\
	ld h, a                 ; > buncha loading going on
	ld a, b                 ;/
	and 7                   ;> mask out the shift number
	ld d, a                 ;> save the shift number
 ld a,(rows)
	ld e, a
FSLoop:                         
	ld a, (ix)              ;> get sprite data
	ld b, a
	inc ix                  ;> point to new sprite data byte
	ld c, 0                 ;> clear c for shift use
	ld a, d                 ;> restore shift # for use
	cp 0
	jp z, NoShift
	ld a, b
	ld b, d
FSShift:
	srl a                   ;\
	rr c                    ; \
	dec b                   ;  > shift until b < 0
	jp z, Shiftend          ; /
	jp FSShift              ;/
NoShift:
	ld a, b
Shiftend:
	xor (hl)                ;> mix the shifted sprite w/screen
	ld (hl), a              ;> and draw it to the screen
	inc hl                  ;> hl points to next screen byte
	ld a, c                 ;> accumulate the remaining shifted data
	xor (hl)                ;> mix 'em
	ld (hl), a              ;> draw it
	ld a, 15                ;\
	add a, l                ; \
	ld l, a                 ;  > Inc the <HL> 1 y value.
	jp nc, FSNoinc          ; /
	inc h                   ;/
FSNoinc:
	dec e                   ; >check if loop is finished

	jp nz, FSLoop           ;/  ---------only have one row-----------
    pop hl                  ;\
	pop ix                  ; \
	pop de                  ;  > mmmpop boop be doop mmmpop
	pop bc                  ; /  
	pop af                  ;/
	ret
FSByte: 
        .db $00


;===========================================================
; RLE picture displayer v1.1
; Decodes a RLE picture made by RLE2PIC
;
; written by David Phillips <electrum@tfs.net>
; started: 8/19/98
; last update: 11/5/98
;
; input: HL = RLE encoded picture, DE = where to display
; output: 1024 byte decoded picture
; destroys: AF, BC, DE, HL
; current size: 32 bytes
;===========================================================
DispRLE:
 ld bc,1024			; we need to copy 
DispRLEL:
 ld a,(hl)			; get the next byte
 cp $91				; is it a run?
 jr z,DispRLERun	; then we need to decode the run
 ldi				; copy the byte, and update counters
DispRLEC:
 ld a,b				; check the low byte and
 or c				; the high byte for 0
 jr nz,DispRLEL		; if not, then we're not done either
 ret				; if it's zero, we're done
DispRLERun:
 inc hl				; move to the run value
 ld a,(hl)			; get the run value
 inc hl				; move to the run count
 push hl			; save source pointer
 ld h,(hl)			; get the run count
 ex de,hl			; swap source and destination pointers
DispRLERunL:
 ld (hl),a			; copy the byte
 inc hl				; increase destination pointer
 dec bc				; decrease byte count
 dec d				; decrease run count
 jr nz,DispRLERunL	; if we're not done, then loop
 ex de,hl			; swap pointers back
 pop hl				; recover source pointer
 inc hl				; advance the source pointer
 jr DispRLEC		; check to see if we should loop


TitleScreen:
; compressed picture made with PIC2RLE
; PIC2RLE by David Phillips <david@acz.org>


 .db $91,$00,$12,$7f,$e0,$00,$00,$0f,$e8,$00,$7f,$00
 .db $08,$91,$00,$05,$03,$82,$1c,$00,$00,$0c,$60,$00
 .db $63,$00,$08,$91,$00,$05,$0c,$02,$03,$00,$00,$0c
 .db $60,$00,$63,$00,$08,$91,$00,$05,$10,$02,$00,$80
 .db $00,$0c,$68,$f0,$63,$3c,$78,$91,$00,$05,$60,$02
 .db $00,$60,$00,$0f,$c9,$10,$7e,$64,$88,$91,$00,$05
 .db $80,$02,$00,$10,$00,$0c,$69,$10,$63,$04,$88,$91
 .db $00,$04,$01,$91,$00,$03,$08,$00,$0c,$69,$10,$63
 .db $7c,$88,$00,$01,$40,$00,$02,$80,$00,$00,$14,$00
 .db $0c,$69,$10,$63,$44,$88,$00,$07,$a0,$00,$02,$40
 .db $00,$00,$24,$00,$0f,$c8,$f6,$7e,$7c,$78,$00,$0f
 .db $f0,$00,$04,$20,$00,$00,$42,$91,$00,$03,$12,$91
 .db $00,$04,$1c,$30,$00,$08,$10,$00,$00,$81,$00,$00
 .db $01,$14,$91,$00,$04,$10,$30,$00,$08,$91,$00,$03
 .db $01,$00,$00,$01,$e0,$91,$00,$04,$10,$10,$00,$10
 .db $91,$00,$04,$80,$91,$00,$07,$16,$d0,$00,$10,$00
 .db $7f,$e0,$00,$80,$91,$00,$07,$16,$d0,$00,$20,$00
 .db $44,$20,$00,$40,$00,$7e,$91,$00,$05,$10,$10,$00
 .db $20,$00,$50,$a0,$00,$40,$00,$63,$91,$00,$05,$11
 .db $10,$00,$20,$00,$7f,$e0,$00,$40,$00,$63,$91,$00
 .db $05,$09,$10,$00,$20,$00,$00,$02,$00,$40,$00,$63
 .db $3c,$79,$c7,$80,$00,$08,$10,$00,$40,$91,$00,$04
 .db $20,$00,$63,$64,$9a,$24,$80,$00,$0b,$a0,$00,$40
 .db $00,$07,$01,$00,$20,$00,$7c,$04,$83,$e4,$00,$00
 .db $08,$20,$00,$40,$00,$0b,$80,$00,$20,$00,$63,$7c
 .db $82,$04,$00,$00,$04,$60,$00,$40,$00,$16,$c2,$00
 .db $20,$00,$63,$44,$8a,$24,$00,$00,$07,$c0,$00,$7e
 .db $00,$1f,$c2,$27,$e0,$00,$63,$7c,$79,$c4,$91,$00
 .db $05,$40,$00,$1b,$42,$20,$20,$91,$00,$0a,$40,$00
 .db $0e,$a4,$40,$20,$91,$00,$0a,$40,$00,$07,$90,$20
 .db $20,$91,$00,$07,$c0,$00,$00,$40,$00,$00,$6c,$40
 .db $20,$00,$00,$03,$80,$fc,$00,$00,$a0,$00,$00,$20
 .db $00,$00,$12,$80,$40,$00,$00,$07,$80,$fc,$00,$00
 .db $ca,$00,$00,$20,$00,$00,$0d,$00,$40,$00,$00,$0d
 .db $80,$c0,$00,$00,$aa,$00,$00,$20,$00,$00,$02,$80
 .db $40,$00,$33,$09,$80,$f8,$00,$00,$c4,$00,$00,$20
 .db $00,$00,$01,$40,$40,$00,$32,$01,$80,$fc,$00,$00
 .db $08,$00,$00,$10,$91,$00,$03,$b0,$80,$00,$12,$01
 .db $80,$0c,$91,$00,$05,$10,$91,$00,$03,$68,$80,$00
 .db $16,$01,$80,$8c,$91,$00,$05,$08,$91,$00,$03,$19
 .db $00,$00,$1c,$01,$88,$fc,$00,$00,$88,$11,$00,$08
 .db $91,$00,$03,$81,$00,$00,$0c,$01,$88,$78,$00,$00
 .db $d9,$91,$91,$01,$00,$04,$08,$00,$00,$42,$91,$00
 .db $08,$aa,$bb,$80,$02,$10,$00,$00,$24,$91,$00,$08
 .db $8a,$91,$91,$01,$00,$02,$20,$00,$00,$14,$91,$00
 .db $08,$89,$99,$80,$01,$40,$00,$00,$08,$91,$00,$0c
 .db $80,$00,$00,$10,$91,$00,$04,$0e,$91,$00,$07,$60
 .db $00,$00,$60,$91,$00,$04,$7b,$91,$00,$03,$f8,$50
 .db $00,$00,$10,$00,$00,$80,$91,$00,$03,$01,$c1,$91
 .db $00,$03,$20,$10,$00,$00,$0c,$00,$03,$91,$00,$04
 .db $0f,$01,$91,$00,$03,$22,$50,$98,$00,$03,$80,$1c
 .db $91,$00,$04,$3c,$01,$91,$00,$03,$25,$55,$54,$00
 .db $00,$7f,$e0,$91,$00,$04,$20,$0f,$e0,$00,$00,$26
 .db $59,$94,$91,$00,$08,$20,$38,$30,$00,$00,$23,$54
 .db $d4,$91,$00,$08,$23,$f0,$0e,$91,$00,$0d,$2e,$10
 .db $03,$91,$00,$0a,$07,$f8,$00,$38,$0c,$01,$80,$91
 .db $00,$09,$1c,$87,$80,$18,$03,$00,$e0,$91,$00,$08
 .db $01,$f1,$80,$f8,$08,$01,$c0,$3f,$91,$00,$08,$01
 .db $f9,$c0,$0b,$ff,$c0,$3f,$e0,$f0,$91,$00,$08,$0c
 .db $1c,$0a,$18,$e0,$00,$00,$18,$91,$00,$08,$1e,$38
 .db $0d,$f0,$60,$00,$00,$08,$91,$00,$08,$70,$0e,$7c
 .db $17,$38,$01,$f0,$08,$91,$00,$08,$7f,$81,$c0,$17
 .db $af,$c3,$08,$08,$91,$00,$08,$01,$06,$00,$17,$a0
 .db $3e,$08,$18,$91,$00,$08,$02,$3c,$00,$13,$20,$06
 .db $e4,$f0,$91,$00,$08,$0f,$e0,$00,$18,$60,$04,$e7
 .db $80,$91,$00,$08,$0f,$00,$00,$0f,$c0,$04,$64,$91
 .db $00,$0e,$03,$08,$91,$00,$0e,$01,$f8,$91,$00,$14

; 1024 bytes compressed to 804 -- 78% of original.

Pic: 
.db %11100111

CarPic: 
.db %11000011 
.db %11111111
.db %11011011
.db %00011000
.db %00100100
.db %11011011
.db %11111111
.db %11000011

Burned1: 
	.db %00100000
	.db %00110010
	.db %00110010
	.db %01010110
	.db %10011111
	.db %10001101
	.db %10000001
	.db %10000001

Burned2: 
	.db %00100000
	.db %00110010
	.db %00110010
	.db %01110110
	.db %11111111
	.db %11101111
	.db %11000111
	.db %11100011



.end
